@@ -15,6 +15,7 @@
 
 import ast
 import re
+import six
 
 import pep8
 
@@ -55,6 +56,10 @@ underscore_import_check_multi = re.compile(r"(.)*import (.)*_, (.)*")
 custom_underscore_check = re.compile(r"(.)*_\s*=\s*(.)*")
 oslo_namespace_imports = re.compile(r"from[\s]*oslo[.](.*)")
 dict_constructor_with_list_copy_re = re.compile(r".*\bdict\((\[)?(\(|\[)")
+assert_no_xrange_re = re.compile(r"\s*xrange\s*\(")
+assert_True = re.compile(r".*assertEqual\(True, .*\)")
+assert_None = re.compile(r".*assertEqual\(None, .*\)")
+no_log_warn = re.compile(r"\s*LOG.warn\(.*")
 
 
 class BaseASTChecker(ast.NodeVisitor):
@@ -119,6 +124,71 @@ def no_translate_debug_logs(logical_line, filename):
         yield(0, "M319 Don't translate debug level logs")
 
 
+class CheckLoggingFormatArgs(BaseASTChecker):
+    """Check for improper use of logging format arguments.
+
+    LOG.debug("Volume %s caught fire and is at %d degrees C and climbing.",
+              ('volume1', 500))
+
+    The format arguments should not be a tuple as it is easy to miss.
+
+    """
+
+    CHECK_DESC = 'M310 Log method arguments should not be a tuple.'
+    LOG_METHODS = [
+        'debug', 'info',
+        'warn', 'warning',
+        'error', 'exception',
+        'critical', 'fatal',
+        'trace', 'log'
+    ]
+
+    def _find_name(self, node):
+        """Return the fully qualified name or a Name or Attribute."""
+        if isinstance(node, ast.Name):
+            return node.id
+        elif (isinstance(node, ast.Attribute)
+                and isinstance(node.value, (ast.Name, ast.Attribute))):
+            method_name = node.attr
+            obj_name = self._find_name(node.value)
+            if obj_name is None:
+                return None
+            return obj_name + '.' + method_name
+        elif isinstance(node, six.string_types):
+            return node
+        else:  # could be Subscript, Call or many more
+            return None
+
+    def visit_Call(self, node):
+        """Look for the 'LOG.*' calls."""
+        # extract the obj_name and method_name
+        if isinstance(node.func, ast.Attribute):
+            obj_name = self._find_name(node.func.value)
+            if isinstance(node.func.value, ast.Name):
+                method_name = node.func.attr
+            elif isinstance(node.func.value, ast.Attribute):
+                obj_name = self._find_name(node.func.value)
+                method_name = node.func.attr
+            else:  # could be Subscript, Call or many more
+                return super(CheckLoggingFormatArgs, self).generic_visit(node)
+
+            # obj must be a logger instance and method must be a log helper
+            if (obj_name != 'LOG'
+                    or method_name not in self.LOG_METHODS):
+                return super(CheckLoggingFormatArgs, self).generic_visit(node)
+
+            # the call must have arguments
+            if not len(node.args):
+                return super(CheckLoggingFormatArgs, self).generic_visit(node)
+
+            # any argument should not be a tuple
+            for arg in node.args:
+                if isinstance(arg, ast.Tuple):
+                    self.add_error(arg)
+
+        return super(CheckLoggingFormatArgs, self).generic_visit(node)
+
+
 def validate_log_translations(logical_line, physical_line, filename):
     # Translations are not required in the test and tempest
     # directories.
@@ -167,40 +237,51 @@ def check_explicit_underscore_import(logical_line, filename):
         yield(0, "M323: Found use of _() without explicit import of _ !")
 
 
-class CheckForStrExc(BaseASTChecker):
-    """Checks for the use of str() on an exception.
+class CheckForStrUnicodeExc(BaseASTChecker):
+    """Checks for the use of str() or unicode() on an exception.
 
-    This currently only handles the case where str() is used in
-    the scope of an exception handler.  If the exception is passed
-    into a function, returned from an assertRaises, or used on an
-    exception created in the same scope, this does not catch it.
+    This currently only handles the case where str() or unicode()
+    is used in the scope of an exception handler.  If the exception
+    is passed into a function, returned from an assertRaises, or
+    used on an exception created in the same scope, this does not
+    catch it.
     """
 
-    CHECK_DESC = ('M325 str() cannot be used on an exception.  '
-                  'Remove or use six.text_type()')
+    CHECK_DESC = ('M325 str() and unicode() cannot be used on an '
+                  'exception.  Remove or use six.text_type()')
 
     def __init__(self, tree, filename):
-        super(CheckForStrExc, self).__init__(tree, filename)
+        super(CheckForStrUnicodeExc, self).__init__(tree, filename)
         self.name = []
         self.already_checked = []
 
+    # Python 2
     def visit_TryExcept(self, node):
         for handler in node.handlers:
             if handler.name:
                 self.name.append(handler.name.id)
-                super(CheckForStrExc, self).generic_visit(node)
+                super(CheckForStrUnicodeExc, self).generic_visit(node)
                 self.name = self.name[:-1]
             else:
-                super(CheckForStrExc, self).generic_visit(node)
+                super(CheckForStrUnicodeExc, self).generic_visit(node)
+
+    # Python 3
+    def visit_ExceptHandler(self, node):
+        if node.name:
+            self.name.append(node.name)
+            super(CheckForStrUnicodeExc, self).generic_visit(node)
+            self.name = self.name[:-1]
+        else:
+            super(CheckForStrUnicodeExc, self).generic_visit(node)
 
     def visit_Call(self, node):
-        if self._check_call_names(node, ['str']):
+        if self._check_call_names(node, ['str', 'unicode']):
             if node not in self.already_checked:
                 self.already_checked.append(node)
                 if isinstance(node.args[0], ast.Name):
                     if node.args[0].id in self.name:
                         self.add_error(node.args[0])
-        super(CheckForStrExc, self).generic_visit(node)
+        super(CheckForStrUnicodeExc, self).generic_visit(node)
 
 
 class CheckForTransAdd(BaseASTChecker):
@@ -243,11 +324,67 @@ def dict_constructor_with_list_copy(logical_line):
         yield (0, msg)
 
 
+def no_xrange(logical_line):
+    if assert_no_xrange_re.match(logical_line):
+        yield(0, "M337: Do not use xrange().")
+
+
+def validate_assertTrue(logical_line):
+    if re.match(assert_True, logical_line):
+        msg = ("M313: Unit tests should use assertTrue(value) instead"
+               " of using assertEqual(True, value).")
+        yield(0, msg)
+
+
+def validate_assertIsNone(logical_line):
+    if re.match(assert_None, logical_line):
+        msg = ("M312: Unit tests should use assertIsNone(value) instead"
+               " of using assertEqual(None, value).")
+        yield(0, msg)
+
+
+def check_uuid4(logical_line):
+    """Generating UUID
+
+    Use oslo_utils.uuidutils to generate UUID instead of uuid4().
+
+    M354
+    """
+
+    msg = ("M354: Use oslo_utils.uuidutils to generate UUID instead "
+           "of uuid4().")
+
+    if "uuid4()." in logical_line:
+        return
+
+    if "uuid4()" in logical_line:
+        yield (0, msg)
+
+
+def no_log_warn_check(logical_line):
+    """Disallow 'LOG.warn'
+
+    Deprecated LOG.warn(), instead use LOG.warning
+    ://bugs.launchpad.net/manila/+bug/1508442
+
+    M338
+    """
+    msg = ("M338: LOG.warn is deprecated, use LOG.warning.")
+    if re.match(no_log_warn, logical_line):
+        yield(0, msg)
+
+
 def factory(register):
     register(validate_log_translations)
     register(check_explicit_underscore_import)
     register(no_translate_debug_logs)
-    register(CheckForStrExc)
+    register(CheckForStrUnicodeExc)
+    register(CheckLoggingFormatArgs)
     register(CheckForTransAdd)
     register(check_oslo_namespace_imports)
     register(dict_constructor_with_list_copy)
+    register(no_xrange)
+    register(validate_assertTrue)
+    register(validate_assertIsNone)
+    register(check_uuid4)
+    register(no_log_warn_check)
