@@ -14,7 +14,7 @@
 #    under the License.
 
 from tempest import config
-from tempest import test
+from testtools import testcase as tc
 
 from manila_tempest_tests.tests.api import base
 from manila_tempest_tests import utils
@@ -29,7 +29,7 @@ class ShareInstancesTest(base.BaseSharesAdminTest):
         super(ShareInstancesTest, cls).resource_setup()
         cls.share = cls.create_share()
 
-    @test.attr(type=["gate", ])
+    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
     def test_get_instances_of_share_v2_3(self):
         """Test that we get only the 1 share instance back for the share."""
         share_instances = self.shares_v2_client.get_instances_of_share(
@@ -47,7 +47,7 @@ class ShareInstancesTest(base.BaseSharesAdminTest):
                                                    self.share['id'],
                                                    si['share_id']))
 
-    @test.attr(type=["gate", ])
+    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
     def test_list_share_instances_v2_3(self):
         """Test that we get at least the share instance back for the share."""
         share_instances = self.shares_v2_client.get_instances_of_share(
@@ -76,6 +76,12 @@ class ShareInstancesTest(base.BaseSharesAdminTest):
             expected_keys.extend(["export_location", "export_locations"])
         if utils.is_microversion_ge(version, '2.10'):
             expected_keys.append("access_rules_status")
+        if utils.is_microversion_ge(version, '2.11'):
+            expected_keys.append("replica_state")
+        if utils.is_microversion_ge(version, '2.22'):
+            expected_keys.append("share_type_id")
+        if utils.is_microversion_ge(version, '2.30'):
+            expected_keys.append("cast_rules_to_readonly")
         expected_keys = sorted(expected_keys)
         actual_keys = sorted(si.keys())
         self.assertEqual(expected_keys, actual_keys,
@@ -83,14 +89,18 @@ class ShareInstancesTest(base.BaseSharesAdminTest):
                          'expected %s, got %s.' % (
                              si['id'], expected_keys, actual_keys))
 
-    @test.attr(type=["gate", ])
+    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
     def test_get_share_instance_v2_3(self):
         self._get_share_instance('2.3')
 
-    @test.attr(type=["gate", ])
+    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
     def test_get_share_instance_v2_9(self):
         self._get_share_instance('2.9')
 
-    @test.attr(type=["gate", ])
+    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
     def test_get_share_instance_v2_10(self):
         self._get_share_instance('2.10')
+
+    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
+    def test_get_share_instance_v2_30(self):
+        self._get_share_instance('2.30')
